'use strict';

const esprima = require('esprima');
const estraverse = require('estraverse');
const escodegen = require('escodegen');
const path = require('path');
const util = require('./util');

const normalizeViewName = util.normalizeViewName;


function updateRouter(options) {
  const content = options.code.toString();
  const ast = esprima.parse(content, { sourceType: 'module' });
  ast.body = addImport(ast.body, options);
  addRoutes(ast, options);
  addRouteFun(ast, options);
  const routerContent = escodegen.generate(ast, {
    format: {
      indent: {
        style: '  ',
        base: 0,
        adjustMultilineComment: false
      }
    }
  });
  return routerContent;
}

function addRouteFun(ast, options) {
  estraverse.traverse(ast, {
    enter: (node) => {
      if (node.type === 'VariableDeclarator' && node.id.name === 'AppRouter') {
        const prop = node.init.arguments[0].properties;

        prop.push({
          'type': 'Property',
          'key': {
            'type': 'Identifier',
            'name': `${normalizeViewName(options.basename, false)}`
          },
          'computed': false,
          'value': {
            'type': 'FunctionExpression',
            'id': null,
            'params': [],
            'body': {
              'type': 'BlockStatement',
              'body': [
                {
                  'type': 'VariableDeclaration',
                  'declarations': [{
                    'type': 'VariableDeclarator',
                    'id': {
                      'type': 'Identifier',
                      'name': `${normalizeViewName(options.basename, false)}View`
                    },
                    'init': {
                      'type': 'NewExpression',
                      'callee': {
                        'type': 'Identifier',
                        'name': `${normalizeViewName(options.basename)}View`
                      },
                      'arguments': []
                    }
                  }],
                  'kind': 'const'
                },
                {
                  'type': 'ExpressionStatement',
                  'expression': {
                    'type': 'CallExpression',
                    'callee': {
                      'type': 'MemberExpression',
                      'computed': false,
                      'object': {
                        'type': 'ThisExpression'
                      },
                      'property': {
                        'type': 'Identifier',
                        'name': 'changeView'
                      }
                    },
                    'arguments': [
                      {
                        'type': 'Identifier',
                        'name': `${normalizeViewName(options.basename, false)}View`
                      }
                    ]
                  }
                }
              ]
            },
            'generator': false,
            'expression': false
          },
          'kind': 'init',
          'method': false,
          'shorthand': false
        });
      }
    }
  });
}

function addRoutes(ast, options) {
  estraverse.traverse(ast, {
    enter: (node) => {
      if (node.type === 'Property' && node.key.name === 'routes') {
        const prop = node.value.properties;

        prop.push({
          'type': 'Property',
          'key': {
            'type': 'Literal',
            'value': normalizeViewName(options.basename, false),
            'raw': `'${normalizeViewName(options.basename, false)}'`
          },
          'computed': false,
          'value': {
            'type': 'Literal',
            'value': normalizeViewName(options.basename, false),
            'raw': `'${normalizeViewName(options.basename, false)}'`
          },
          'kind': 'init',
          'method': false,
          'shorthand': false
        });
      }
    }
  });
}

function addImport(body, options) {
  let lastImport = 0;

  for (let i = 0; i < body.length; i++) {
    if (body[i].type === 'ImportDeclaration') {
      lastImport = i;
    }
  }

  const newbody = [];

  for (let i = 0; i <= lastImport; i++) {
    newbody.push(body[i]);
  }
  newbody.push({
    'type': 'ImportDeclaration',
    'specifiers': [
      {
        'type': 'ImportDefaultSpecifier',
        'local': {
          'type': 'Identifier',
          'name': `${normalizeViewName(options.basename)}View`
        }
      }
    ],
    'source': {
      'type': 'Literal',
      'value': `./views/${path.join(path.dirname(options.name), normalizeViewName(options.basename))}`,
      'raw': `'./views/${path.join(path.dirname(options.name), normalizeViewName(options.basename))}'`
    }
  });

  for (let i = lastImport + 1; i < body.length; i++) {
    newbody.push(body[i]);
  }

  return newbody;
}

module.exports = updateRouter;
