'use strict';

/**
 * 文件相关操作
 * 扩展fse包再export
 */
'use strict';
const path = require('path');
const fs = require('fs');
const fse = require('fs-extra');

const CWD = process.cwd();

/**
 * 是否是安全的目标路径
 * 目标路径为空或者为空目录
 * @param  {String}  dest
 * @return {Boolean}
 */
function isSafeDest(dest) {
  return !fs.existsSync(dest) || (fs.statSync(dest).isDirectory() && !fs.readdirSync(dest).length);
}

/**
 * 从arr中按顺序查找
 */
function getExistsPath(arr) {
  let ret = '';

  arr.some((p) => {
    p = path.join(CWD, p);
    if (fse.existsSync(p)) {
      ret = p;
      return true;
    }
    return false;
  });

  return ret;
}

function moveSync(src, target) {
  fse.removeSync(target);
  fse.copySync(src, target);
  fse.removeSync(src);
}

fse.getExistsPath = getExistsPath;
fse.isSafeDest = isSafeDest;
fse.moveSync = moveSync;

module.exports = fse;
