'use strict';

const path = require('path');

/**
 * 简单模板替换
 */
function compile(template) {
  return function (context) {
    return template.replace(/\\?\{\{\s*([^{}\s]+)\s*\}\}/g, (match, name) => {
      return (context[name] == null) ? match : context[name];
    });
  };
}

/**
 * 首字母大写
 */
function ucfirst(str) {
  return str.charAt(0).toUpperCase() + str.substring(1);
}

function lcfirst(str) {
  return str.charAt(0).toLowerCase() + str.substring(1);
}

/**
 * 转为驼峰
 */
function camelize(str) {
  if (str.indexOf('-') < 0 && str.indexOf('_') < 0) {
    return str;
  }
  return str.replace(/[-_][^-_]/g, (match) => match.charAt(1).toUpperCase());
}

module.exports = {
  ucfirst,
  lcfirst,
  compile,

  normalizeViewName: (name, upper) => {
    // upper默认为true
    upper = typeof upper === 'undefined' ? true : upper;

    let dir = path.dirname(name);
    if (dir === '.') {
      dir = '';
    }

    name = camelize(path.basename(name));
    return path.join(dir, (upper ? ucfirst(name) : lcfirst(name)));
  }
};
