'use strict';

const path = require('path');
const chalk = require('chalk');
const fs = require('fs');
const glob = require('glob');
const mkdirp = require('mkdirp');
const fse = require('../lib/fse');
const util = require('../lib/util');
const autoRouter = require('../lib/autoRouter');

const REG_VIEW = /\$\$View/g;
const REG_VIEW_LOW = /\$\$ViewLow/g;
const tplDir = path.resolve(__dirname, '..', 'view');

module.exports = function (viewName) {
  const viewsPath = fse.getExistsPath(['src/views', 'views']);
  if (!viewsPath) {
    console.log(chalk.red('未找到src/views或views，请确认当前目录是否正确'));
    process.exit(0);
  }

  const name = util.normalizeViewName(viewName);
  const dest = path.resolve(viewsPath, name);

  if (!fse.isSafeDest(dest)) {
    console.log(chalk.red('文件已存在或不为空目录'));
    process.exit(0);
  }

  console.log('创建文件..');
  mkdirp.sync(dest);
  glob.sync('**', {
    cwd: tplDir,
    nodir: false
  }).forEach((source) => {
    const fileName = source.replace(REG_VIEW, path.basename(name));
    const target = path.join(dest, fileName);
    source = path.join(tplDir, source);
    console.log(target);
    if (target.indexOf(path.sep) !== -1) {
      mkdirp.sync(path.dirname(target));
    }
    if (fs.statSync(source).isDirectory() === true) {
      // 空文件夹
      mkdirp.sync(target);
      return;
    }
    let contents = fs.readFileSync(source, 'utf-8');
    const depth = name.split('/').length - 1;
    if (depth) {
      contents = contents.replace(/(\.\.\/)+/g, `$&${new Array(depth + 1).join('../')}`);
    }
    contents = contents.replace(REG_VIEW_LOW, path.basename(name).toLocaleLowerCase());
    contents = contents.replace(REG_VIEW, path.basename(name));
    fs.writeFileSync(target, contents);
  });

  /*
   * 自动修改router文件
   * 需要一个require
   * 修改router及对应的handler
   */
  const routerFile = fse.getExistsPath(['src/appRouter.js', 'appRouter.js', 'src/app.js', 'app.js']);
  if (routerFile) {
    const code = autoRouter({
      name: viewName,
      basename: path.basename(viewName),
      code: fse.readFileSync(routerFile, 'utf-8')
    });
    fse.writeFileSync(routerFile, code);
  }

  console.log(`创建成功，文件目录为：${chalk.yellow(dest)}`);
};
